CON
  _clkmode        = xtal1 + pll16x
  _xinfreq        = 5_000_000


VAR
  long SrvCh1Pin
  long SrvCh2Pin
  long SrvCh3Pin
  long SrvCh4Pin
  long SrvCh5Pin
  long SrvCh6Pin
  long SrvCh7Pin
  long SrvCh8Pin

  long SrvCh1TickDiff
  long SrvCh2TickDiff
  long PulseLength_microsecs

  long PulseLength

OBJ
  _vp    : "Conduit"                        'transfers data to/from PC
  SERVO  : "Servo32v7"
  dbg    : "PASDebug"                '<---- Add for Debugger 


Pub Main

  'tell EZLog the variable names
  'the variablenames should ALWAYS be the same as in the var section
  'to keep things clear
  _vp.config(string("var:SrvCh1TickDiff,SrvCh2TickDiff,PulseLength_microsecs")) 

  'share variables Btn1Cnt..S_Width with EZLog
  '@variablename means HUB-RAM adress of variable 
  _vp.share(@SrvCh1TickDiff,@PulseLength_microsecs)                 

  SERVO.Start
  SERVO.Set(9,1500)

  SrvCh1TickDiff := 128
  WaitCnt(ClkFreq * 5 + cnt)
  Start (10,11,12,13,14,15,16,17)
  repeat
    repeat PulseLength from 1600 to 1700 step 10
      SERVO.Set(9,PulseLength)
      PulseLength_microsecs := SrvCh1TickDiff / 8
      WaitCnt(ClkFreq * 2 + cnt)

  
PUB Start (p_SrvCh1, p_SrvCh2, p_SrvCh3, p_SrvCh4, p_SrvCh5, p_SrvCh6, p_SrvCh7, p_SrvCh8)   

  SrvCh1Pin := p_SrvCh1 
  SrvCh2Pin := p_SrvCh2 
  SrvCh3Pin := p_SrvCh3 
  SrvCh4Pin := p_SrvCh4 
  SrvCh5Pin := p_SrvCh5 
  SrvCh6Pin := p_SrvCh6 
  SrvCh7Pin := p_SrvCh7 
  SrvCh8Pin := p_SrvCh8 

  cognew(@entry, @SrvCh1Pin)
  'dbg.start(31,30,@entry)                 '<---- Add for Debugger

  'repeat 'this repeat is nescessary because FullDuplexSerial
         'on the same IO-pins disturbes the communication with PASD
         'for PASD it is enough that the debug-kernel is running with
         'PASM-code 

CON
  Flag_cleared = 1

  
DAT
                        org     0
                        fit     496
entry
{
'  --------- Debugger Kernel add this at Entry (Addr 0) ---------
   long $34FC1202,$6CE81201,$83C120B,$8BC0E0A,$E87C0E03,$8BC0E0A
   long $EC7C0E05,$A0BC1207,$5C7C0003,$5C7C0003,$7FFC,$7FF8
'  --------------------------------------------------------------
} 
                      mov     _Ptr2HUB_RAM_var,par           ' par contains the value of the second cognew-parameter
                                                             ' in this object originally "cognew(@entry, @SrvCh1Pin)"
                                                             ' this means par contains the ADRESS of HUB-RAM variable SrvCh1Pin
                                                             ' this adress is copied to COG-RAM-cell labeled "_Ptr2HUB_RAM_var"
                                                             ' _Ptr2HUB_RAM_var is just a copy of par and keeps par unchanged for later uses.
                                                             
                      rdlong    _SrvCh1Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh1Pin"
                      mov       _SrvCh1PinMask, #1           'load COG-RAM-cell labeled "_SrvCh1PinMask" with value 1
                                                             'this means _SrvCh1PinMask contains %00000000_00000000_00000000_00000001 (LSB is 1) 
                      shl       _SrvCh1PinMask, _SrvCh1Pin   'shift the LSB to the left. _SrvCh1Pin contains number of bitpositions to shift left

'example: _SrvCh1Pin contains value 3                 shl       _SrvCh1PinMask, _SrvCh1Pin
'         which means                                 shl       _SrvCh1PinMask, 3
'before executing _SrvCh1PinMask contains %00000000_00000000_00000000_00000001
'after  executing _SrvCh1PinMask contains %00000000_00000000_00000000_00001000 
'                                                                         321 << shifting 3 bits to the left
'explantion for the add-command below
'add direct value 4 to _Ptr2HUB_RAM_var as each long is 4 bytes this means _Ptr2HUB_RAM_var contains the adress of the     
'HUB-RAM-variable that comes next after the first one (first one here is "SrvCh1Pin") second one is "SrvCh2Pin"
'ATTENTION COG-RAM-cells have similar names but begin with an underline "_"
                      
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh2Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh2Pin"  
                      mov       _SrvCh2PinMask, #1           'load COG-RAM-cell labeled "_SrvCh2PinMask "with value (1 LSB is one)
                      shl       _SrvCh2PinMask, _SrvCh2Pin   'shift the LSB to the left. 

                      
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh3Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh3Pin"  
                      mov       _SrvCh3PinMask, #1           'load COG-RAM-cell labeled "_SrvCh3PinMask "with value (1 LSB is one)
                      shl       _SrvCh3PinMask, _SrvCh3Pin   'shift the LSB to the left. 
                                                                
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh4Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh4Pin"  
                      mov       _SrvCh4PinMask, #1           'load COG-RAM-cell labeled "_SrvCh4PinMask "with value (1 LSB is one)
                      shl       _SrvCh4PinMask, _SrvCh4Pin   'shift the LSB to the left. 
                                                                
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh5Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh5Pin"  
                      mov       _SrvCh5PinMask, #1           'load COG-RAM-cell labeled "_SrvCh5PinMask "with value (1 LSB is one)
                      shl       _SrvCh5PinMask, _SrvCh5Pin   'shift the LSB to the left. 
                                                                
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh6Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh6Pin"  
                      mov       _SrvCh6PinMask, #1           'load COG-RAM-cell labeled "_SrvCh6PinMask "with value (1 LSB is one)
                      shl       _SrvCh6PinMask, _SrvCh6Pin   'shift the LSB to the left. 
                                                                
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh7Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh7Pin"  
                      mov       _SrvCh7PinMask, #1           'load COG-RAM-cell labeled "_SrvCh7PinMask "with value (1 LSB is one)
                      shl       _SrvCh7PinMask, _SrvCh7Pin   'shift the LSB to the left. 
                                                                
                      add       _Ptr2HUB_RAM_var, #4         
                      rdlong    _SrvCh8Pin,_Ptr2HUB_RAM_var  'load value at adress _Ptr2HUB_RAM_var into COG-RAM-cell labeled "_SrvCh8Pin"  
                      mov       _SrvCh8PinMask, #1           'load COG-RAM-cell labeled "_SrvCh8PinMask "with value (1 LSB is one)
                      shl       _SrvCh8PinMask, _SrvCh7Pin   'shift the LSB to the left. 
                      

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh1TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh2TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh3TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh4TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh5TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh6TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh7TickDiff, _Ptr2HUB_RAM_var   

                      add       _Ptr2HUB_RAM_var, #4
                      mov       _Ptr2SrvCh8TickDiff, _Ptr2HUB_RAM_var   


                      'initialise COG-RAM-cells used for systemcounter-snapshots with value Flag_cleared
                      mov      _SrvCh1_low_high_cnt, #Flag_cleared
                      mov      _SrvCh2_low_high_cnt, #Flag_cleared
                      mov      _SrvCh3_low_high_cnt, #Flag_cleared
                      mov      _SrvCh4_low_high_cnt, #Flag_cleared
                      mov      _SrvCh5_low_high_cnt, #Flag_cleared
                      mov      _SrvCh6_low_high_cnt, #Flag_cleared
                      mov      _SrvCh7_low_high_cnt, #Flag_cleared
                      mov      _SrvCh8_low_high_cnt, #Flag_cleared

                      mov      _SrvCh1_high_low_cnt, #Flag_cleared
                      mov      _SrvCh2_high_low_cnt, #Flag_cleared
                      mov      _SrvCh3_high_low_cnt, #Flag_cleared
                      mov      _SrvCh4_high_low_cnt, #Flag_cleared
                      mov      _SrvCh5_high_low_cnt, #Flag_cleared
                      mov      _SrvCh6_high_low_cnt, #Flag_cleared
                      mov      _SrvCh7_high_low_cnt, #Flag_cleared
                      mov      _SrvCh8_high_low_cnt, #Flag_cleared
                                                                
:polling_Pins         mov       _PinStates,INA               'copy input-pinregister into COG-RAM-cell labeled _PinStates

                      cmp      _SrvCh1_low_high_cnt, #Flag_cleared wz   'set z-flag to 1 if _SrvCh1_low_high_cnt is cleared (to value Flag_cleared)
                                                             'this indicates that Servochannel 1 is in the state of waiting for a low-high-transition
                                                             'after the low-high-transistion is detected _SrvCh1_low_high_cnt
                                                             'contains the snapshot of the systemcounter cnt

         if_z         call      #Check_for_low_high1

                      cmp      _SrvCh1_low_high_cnt, #Flag_cleared wz   'set z-flag to 1 if _SrvCh1_low_high_cnt is cleared (to 1)
         if_nz        call      #Check_for_high_low1                    'if z-flag is 0 a low-high-transition was detected so start checking for a high-low-transition                    



                      cmp      _SrvCh2_low_high_cnt, #Flag_cleared wz   'set z-flag to 1 if _SrvCh1_low_high_cnt is cleared (to value Flag_cleared)
         if_z         call      #Check_for_low_high2

                      cmp      _SrvCh2_low_high_cnt, #Flag_cleared wz   'set z-flag to 1 if _SrvCh1_low_high_cnt is cleared (to 1)
         if_nz        call      #Check_for_high_low2                    'if z-flag is 0 a low-high-transition was detected so start checking for a high-low-transition                    


                      jmp       #:polling_Pins
                      



Check_for_low_high1 '***********************************************************
                      Test      _SrvCh1PinMask, ina    wz     'if low-high-transition is not yet detected (z-flag is set ) tests and sets z= 0 if both = 1
         if_nz        mov       _SrvCh1_low_high_cnt, cnt     'if z-flag is NOT set store value of systemcounter in COG-RAM-cell labeled _SrvCh1_low_high_cnt
                      'mov       _SrvCh1_high_low_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_high_low_cnt" meaning
                                                              'cnt-value of last last high_low-transition is no longer valid as a new low-high-transition is detected  
Check_for_low_high1_ret ret '###################################################



Check_for_high_low1 '***********************************************************
                      Test      _SrvCh1PinMask, ina    wz     'if low-high-transition is not yet detected (z-flag is set ) tests and sets z= 0 if both = 1
         if_z         mov       _SrvCh1_high_low_cnt, cnt     'if z-flag is set store value of systemcounter in COG-RAM-cell labeled _SrvCh1_low_high_cnt

                      cmp      _SrvCh1_high_low_cnt, #Flag_cleared wz 'set z-flag to 1 if _SrvCh1_high_low_cnt is still cleared (to value Flag_cleared)   
         if_nz        call     #CalculateTickDiff1                     'if z-flag is 0 which means _SrvCh1_high_low_cnt <> Flag_cleared a high-low-transition is detected
                                                                      'pulse has finished go calculating the pulselength 
Check_for_high_low1_ret ret'###################################################



CalculateTickDiff1 '***********************************************************                       
                      mov       _SrvCh1_tick_diff, _SrvCh1_high_low_cnt     'load value of _SrvCh1_high_low_cnt into _SrvCh1_tick_diff 
                      sub       _SrvCh1_tick_diff, _SrvCh1_low_high_cnt WC  'substract _SrvCh1_low_high_cnt from _SrvCh1_tick_diff
                                                                            'as _SrvCh1_tick_diff contains value of _SrvCh1_high_low_cnt
                                                                            'after executing this command _SrvCh1_tick_diff = _SrvCh1_high_low_cnt - _SrvCh1_low_high_cnt
                                                                            'which means _SrvCh1_tick_diff contains really the pulselength in systemcount-ticks
                                                                            
         if_c         call      #Handle_cnt_Rollover1                                                              

                      'mov       _SrvCh1_low_high_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_low_high_cnt" meaning
                                                              'high-pulse as a low-high-low-transition-SEQUENCE has been detected
                                                              'and _SrvCh1_low_high_cnt = 0 indicates check for a new low-high-transition
                                                               
                      mov       _SrvCh1_high_low_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_high_low_cnt" meaning
                      mov       _SrvCh1_low_high_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_high_low_cnt" meaning


                      wrlong    _SrvCh1_tick_diff,_Ptr2SrvCh1TickDiff
                      
CalculateTickDiff1_ret ret'###################################################


Handle_cnt_Rollover1 '***********************************************************
'if the systtemcounter cnt had an rollover from max to zero
'between the low-high and the high-low-transition
'the correct calculation if the tick-difference is  
'$FFFFFFFF -  _SrvCh1_low_high_cnt + _SrvCh1_high_low_cnt

                      mov _SrvCh1_tick_diff, _all_bits_one    'load with max-value all 32 bits set 1
                      
                      sub _SrvCh1_tick_diff, _SrvCh1_low_high_cnt 
                      add _SrvCh1_tick_diff, _SrvCh1_high_low_cnt

Handle_cnt_Rollover1_ret ret'###################################################



Check_for_low_high2  '***********************************************************
                      Test      _SrvCh2PinMask, ina    wz     'if low-high-transition is not yet detected (z-flag is set ) tests and sets z= 0 if both = 1
         if_nz        mov       _SrvCh2_low_high_cnt, cnt     'if z-flag is NOT set store value of systemcounter in COG-RAM-cell labeled _SrvCh1_low_high_cnt
                      'mov       _SrvCh1_high_low_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_high_low_cnt" meaning
                                                              'cnt-value of last last high_low-transition is no longer valid as a new low-high-transition is detected  
Check_for_low_high2_ret ret'###################################################



Check_for_high_low2 '***********************************************************
                      Test      _SrvCh2PinMask, ina    wz     'if low-high-transition is not yet detected (z-flag is set ) tests and sets z= 0 if both = 1
         if_z         mov       _SrvCh2_high_low_cnt, cnt     'if z-flag is set store value of systemcounter in COG-RAM-cell labeled _SrvCh1_low_high_cnt

                      cmp      _SrvCh2_high_low_cnt, #Flag_cleared wz 'set z-flag to 1 if _SrvCh1_high_low_cnt is still cleared (to value Flag_cleared)   
         if_nz        call     #CalculateTickDiff2                     'if z-flag is 0 which means _SrvCh1_high_low_cnt <> Flag_cleared a high-low-transition is detected
                                                                      'pulse has finished go calculating the pulselength 
Check_for_high_low2_ret ret'###################################################



CalculateTickDiff2 '***********************************************************                       
                      mov       _SrvCh2_tick_diff, _SrvCh2_high_low_cnt     'load value of _SrvCh1_high_low_cnt into _SrvCh1_tick_diff 
                      sub       _SrvCh2_tick_diff, _SrvCh2_low_high_cnt WC  'substract _SrvCh1_low_high_cnt from _SrvCh1_tick_diff
                                                                            'as _SrvCh1_tick_diff contains value of _SrvCh1_high_low_cnt
                                                                            'after executing this command _SrvCh1_tick_diff = _SrvCh1_high_low_cnt - _SrvCh1_low_high_cnt
                                                                            'which means _SrvCh1_tick_diff contains really the pulselength in systemcount-ticks
                                                                            
         if_c         call      #Handle_cnt_Rollover2                                                              

                      'mov       _SrvCh1_low_high_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_low_high_cnt" meaning
                                                              'high-pulse as a low-high-low-transition-SEQUENCE has been detected
                                                              'and _SrvCh1_low_high_cnt = 0 indicates check for a new low-high-transition
                                                               
                      mov       _SrvCh2_high_low_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_high_low_cnt" meaning
                      mov       _SrvCh2_low_high_cnt, #Flag_cleared      'clear COG-RAM-cell labeled "_SrvCh1_high_low_cnt" meaning


                      wrlong    _SrvCh2_tick_diff,_Ptr2SrvCh2TickDiff
                      
CalculateTickDiff2_ret ret'###################################################


Handle_cnt_Rollover2 '***********************************************************
'if the systtemcounter cnt had an rollover from max to zero
'between the low-high and the high-low-transition
'the correct calculation if the tick-difference is  
'$FFFFFFFF -  _SrvCh1_low_high_cnt + _SrvCh1_high_low_cnt

                      mov _SrvCh2_tick_diff, _all_bits_one    'load with max-value all 32 bits set 1
                      
                      sub _SrvCh2_tick_diff, _SrvCh2_low_high_cnt 
                      add _SrvCh2_tick_diff, _SrvCh2_high_low_cnt

Handle_cnt_Rollover2_ret ret'###################################################



         

'COG-RAM "variables"

_all_bits_one         long    $FFFFFFFF                                                                

_Ptr2HUB_RAM_var      res     1

_PinStates            res     1

_Ptr2SrvCh1TickDiff   res     1
_Ptr2SrvCh2TickDiff   res     1
_Ptr2SrvCh3TickDiff   res     1
_Ptr2SrvCh4TickDiff   res     1
_Ptr2SrvCh5TickDiff   res     1
_Ptr2SrvCh6TickDiff   res     1
_Ptr2SrvCh7TickDiff   res     1
_Ptr2SrvCh8TickDiff   res     1


_SrvCh1Pin            res     1                                                               
_SrvCh2Pin            res     1                                                               
_SrvCh3Pin            res     1                                                               
_SrvCh4Pin            res     1                                                               
_SrvCh5Pin            res     1                                                               
_SrvCh6Pin            res     1                                                               
_SrvCh7Pin            res     1                                                               
_SrvCh8Pin            res     1


_SrvCh1PinMask        res     1                                                               
_SrvCh2PinMask        res     1                                                               
_SrvCh3PinMask        res     1                                                               
_SrvCh4PinMask        res     1                                                               
_SrvCh5PinMask        res     1                                                               
_SrvCh6PinMask        res     1                                                               
_SrvCh7PinMask        res     1                                                               
_SrvCh8PinMask        res     1                                                               


_SrvCh1_low_high_cnt  res     1
_SrvCh2_low_high_cnt  res     1
_SrvCh3_low_high_cnt  res     1
_SrvCh4_low_high_cnt  res     1
_SrvCh5_low_high_cnt  res     1
_SrvCh6_low_high_cnt  res     1
_SrvCh7_low_high_cnt  res     1
_SrvCh8_low_high_cnt  res     1


_SrvCh1_high_low_cnt  res     1
_SrvCh2_high_low_cnt  res     1
_SrvCh3_high_low_cnt  res     1
_SrvCh4_high_low_cnt  res     1
_SrvCh5_high_low_cnt  res     1
_SrvCh6_high_low_cnt  res     1
_SrvCh7_high_low_cnt  res     1
_SrvCh8_high_low_cnt  res     1

_SrvCh1_tick_diff     res     1
_SrvCh2_tick_diff     res     1
_SrvCh3_tick_diff     res     1
_SrvCh4_tick_diff     res     1
_SrvCh5_tick_diff     res     1
_SrvCh6_tick_diff     res     1
_SrvCh7_tick_diff     res     1
_SrvCh8_tick_diff     res     1


_SvrCh1_measure_hightime res 1
_SvrCh2_measure_hightime res 1
_SvrCh3_measure_hightime res 1
_SvrCh4_measure_hightime res 1
_SvrCh5_measure_hightime res 1
_SvrCh6_measure_hightime res 1
_SvrCh7_measure_hightime res 1
_SvrCh8_measure_hightime res 1

                                                               
_finit_loop_cntr      res     1 'used in finite loops as counter-variable                      

CRV_for_ParPointer    res     1 'CRV stands for C)ogR)AM-V)ariable
CRV_value             res     1 


